#pragma pack(2)
#include <Common.h>
#include <System/SysAll.h>
#include <System/NetMgr.h>
#include <System/AudioLib.h>
#include <UI/UIAll.h>

#include "fonehome.h"
#include "res.h"

static int StartApplication(void);
static Boolean OpenDatabase(void);
static void EventLoop(void);
static void StopApplication(void);
static Boolean MainEventHandler(EventPtr event);
static void PlayNote(DWord);
static void PlayCustom(Int);
static void DoCustom();
static void DrawCustomButtons(FormPtr form);
static void DoAbout();
static void PlayClip();
static void DoAddressDial();

int err;
ListPtr SpeedListPtr;
Word libRef;
Boolean TRGPro=false;
Boolean debug=false;
DmOpenRef customDB;

DWord  PilotMain (Word cmd, Ptr cmdPBP, Word launchFlags)
	{
	int error;

	if ( cmd == sysAppLaunchCmdNormalLaunch )
		{

		error = OpenDatabase();
		if ( error ) return error;
		error = StartApplication();  // Application start code
		if ( error ) return error;

		EventLoop();  // Event loop

		StopApplication (); // Application stop code
		}
	return 0;
	}

static Boolean OpenDatabase()
{
	Err error;
	int i;

	customDB=DmOpenDatabaseByTypeCreator('FHnm', creatID, dmModeReadWrite);
	if (!customDB)
	{
		if (DmGetLastErr()==dmErrCantFind)
		{
			error=DmCreateDatabase(0, "FoneHome numbers", creatID, 'FHnm', false);
			if (error) return error;
			customDB=DmOpenDatabaseByTypeCreator('FHnm', creatID, dmModeReadWrite);
			if (!customDB) return 1;
		}
		for (i=1;i<7;i++)
		{
			VoidHand newrechand;
			Char fieldname[13];
			customrec *prot, working;
			UInt index=dmMaxRecordIndex;
			
			newrechand=DmNewRecord(customDB, &index, sizeof(customrec));
			prot=(customrec *)MemHandleLock(newrechand);
			StrPrintF(fieldname, "Custom %d", i);
			MemSet(&working, sizeof(working), 0);
			StrCopy(working.name, fieldname);
			DmWrite(prot, 0, &working, sizeof(working));
			MemHandleUnlock(newrechand);
			DmReleaseRecord(customDB, index, true);
		}
	}
	return 0;
}

static int StartApplication(void)
	{

	FrmGotoForm(formID_main);
	return 0;

	}

static void StopApplication(void)
	{

	if (TRGPro) AudioLibClose(libRef);
	if (TRGPro) SysLibRemove(libRef);
	if (customDB) DmCloseDatabase(customDB);
	while ( 0 );

	}

static void EventLoop(void)
	{
	//short err;
	int formID;
	FormPtr form;
	EventType event;
	//Word fish;

	do
		{

		EvtGetEvent(&event, 200);

		if ( SysHandleEvent(&event) )	continue;
		if ( MenuHandleEvent((void *)0, &event, (short *)&err)) continue;

		if ( event.eType == frmLoadEvent )
			{
			formID = event.data.frmLoad.formID;
			form = FrmInitForm(formID);
			FrmSetActiveForm(form);
			switch ( formID )
				{
				case formID_main:
					FrmSetEventHandler(form, (FormEventHandlerPtr) MainEventHandler);
					break;
				}
			}
		FrmDispatchEvent(&event);
		}while ( event.eType != appStopEvent );
	}

static Boolean MainEventHandler(EventPtr event)
	{
	FormPtr   form;
	int       handled = 0;

	switch ( event->eType )
		{
		case frmOpenEvent:
			form = FrmGetActiveForm();
	
			err = SysLibFind(AudioLibName, &libRef);

			if (err)
			{
				err = SysLibLoad(AudioLibTypeID, AudioLibCreatorID, &libRef);
				if (err==sysErrLibNotFound)
				{
					FrmAlert(formID_notaTRGPro);
					TRGPro=false;
					err=false;
				}
				else if (err)
				{
					return 1;
				}
				else
				{
					TRGPro=true;
				}
			}
			else
			{
				TRGPro=true;
			}

			if (TRGPro) err = AudioLibOpen(libRef);

			if (err)
			{
				return 1;
			}
			DrawCustomButtons(form);
		FrmDrawForm(form);
			handled = 1;
			break;

		case ctlSelectEvent:	 // A control button was pressed and released.
			if (( event->data.ctlEnter.controlID>=buttonID_0 ) && ( event->data.ctlEnter.controlID<=buttonID_STAR ))
			{
				PlayNote(event->data.ctlEnter.controlID);
				handled = 1;
			}
			else if (( event->data.ctlEnter.controlID>=buttonID_custom1 ) && ( event->data.ctlEnter.controlID<=buttonID_custom6 ))
			{
				PlayCustom((event->data.ctlEnter.controlID)-custombase);
				handled = 1;
			}
			else if ( event->data.ctlEnter.controlID==buttonID_clip)
			{
				PlayClip();
				handled=1;
			}
			else if ( event->data.ctlEnter.controlID==buttonID_address)
			{
				DoAddressDial();
				handled=1;
			}
			break;
		case menuEvent:
			switch (event->data.menu.itemID)
			{
				case menuID_custom:
					DoCustom();
					handled=1;
					break;
				case menuID_about:
					DoAbout();
					handled=1;
					break;
				case menuID_clipboard:
					PlayClip();
					handled=1;
					break;
				default:
					handled=0;
					break;
			}
			break;
		case nilEvent:
			handled = 1;
			break;
		default:
			handled = 0;
			break;
		}
	return handled;
}

static void PlayNote(DWord notename)
{

	switch (notename)
	{
		case buttonID_1:
			if (TRGPro) AudioPlayDTMFChar(libRef, '1', 2);
			if (debug) WinDrawChars("1  ", 3, 20, 140);
			break;
		case buttonID_2:
			if (TRGPro) AudioPlayDTMFChar(libRef, '2', 2);
			if (debug) WinDrawChars("2  ", 3, 20, 140);
			break;
		case buttonID_3:
			if (TRGPro) AudioPlayDTMFChar(libRef, '3', 2);
			if (debug) WinDrawChars("3  ", 3, 20, 140);
			break;
		case buttonID_4:
			if (TRGPro) AudioPlayDTMFChar(libRef, '4', 2);
			if (debug) WinDrawChars("4  ", 3, 20, 140);
			break;
		case buttonID_5:
			if (TRGPro) AudioPlayDTMFChar(libRef, '5', 2);
			if (debug) WinDrawChars("5  ", 3, 20, 140);
			break;
		case buttonID_6:
			if (TRGPro) AudioPlayDTMFChar(libRef, '6', 2);
			if (debug) WinDrawChars("6  ", 3, 20, 140);
			break;
		case buttonID_7:
			if (TRGPro) AudioPlayDTMFChar(libRef, '7', 2);
			if (debug) WinDrawChars("7  ", 3, 20, 140);
			break;
		case buttonID_8:
			if (TRGPro) AudioPlayDTMFChar(libRef, '8', 2);
			if (debug) WinDrawChars("8  ", 3, 20, 140);
			break;
		case buttonID_9:
			if (TRGPro) AudioPlayDTMFChar(libRef, '9', 2);
			if (debug) WinDrawChars("9  ", 3, 20, 140);
			break;
		case buttonID_0:
			if (TRGPro) AudioPlayDTMFChar(libRef, '0', 2);
			if (debug) WinDrawChars("0  ", 3, 20, 140);
			break;
		case buttonID_A:
			if (TRGPro) AudioPlayDTMFChar(libRef, 'A', 2);
			if (debug) WinDrawChars("A  ", 3, 20, 140);
			break;
		case buttonID_B:
			if (TRGPro) AudioPlayDTMFChar(libRef, 'B', 2);
			if (debug) WinDrawChars("B  ", 3, 20, 140);
			break;
		case buttonID_C:
			if (TRGPro) AudioPlayDTMFChar(libRef, 'C', 2);
			if (debug) WinDrawChars("C  ", 3, 20, 140);
			break;
		case buttonID_D:
			if (TRGPro) AudioPlayDTMFChar(libRef, 'D', 2);
			if (debug) WinDrawChars("D  ", 3, 20, 140);
			break;
		case buttonID_STAR:
			if (TRGPro) AudioPlayDTMFChar(libRef, '*', 2);
			if (debug) WinDrawChars("*  ", 3, 20, 140);
			break;
		case buttonID_HASH:
			if (TRGPro) AudioPlayDTMFChar(libRef, '#', 2);
			if (debug) WinDrawChars("#  ", 3, 20, 140);
			break;
		default:
			break;
	}

}

static void PlayCustom(Int custnum)
{
		VoidHand myhand=DmQueryRecord(customDB, custnum-1);
		customrec *working=MemHandleLock(myhand);
		if (TRGPro)
		{
			if (StrLen(working->number)>0)
			{
				AudioPlayDTMFStr(libRef, working->number, 2, 3);
			}
		}
		MemHandleUnlock(myhand);
}

static void DoCustom()
{
	FormPtr previousForm= FrmGetActiveForm();
	FormPtr form = FrmInitForm(formID_custom);
	Word	hitButton;
	int 	i;

	FrmSetActiveForm(form);
	for (i=1;i<7;i++)
	{
		FieldPtr	fldP;
		Handle		txtH;
		VoidHand	myhand=DmQueryRecord(customDB, i-1);
		customrec	*working=MemHandleLock(myhand);
		VoidHand	nameh,numh;

		nameh=MemHandleNew(StrLen(working->name)+1);
		numh=MemHandleNew(StrLen(working->number)+1);
		StrCopy(MemHandleLock(nameh),working->name);
		StrCopy(MemHandleLock(numh),working->number);
		MemHandleUnlock(nameh);
		MemHandleUnlock(numh);
		MemHandleUnlock(myhand);
		fldP=FrmGetObjectPtr(form, FrmGetObjectIndex(form, 4000+i));
		txtH=FldGetTextHandle(fldP);
		FldSetTextHandle(fldP, (Handle)nameh);
		if (txtH) MemHandleFree((VoidHand)txtH);
		fldP=FrmGetObjectPtr(form, FrmGetObjectIndex(form, 4010+i));
		txtH=FldGetTextHandle(fldP);
		FldSetTextHandle(fldP, (Handle)numh);
		if (txtH) MemHandleFree((VoidHand)txtH);
	}
	hitButton=FrmDoDialog(form);
	if (hitButton==buttonID_customOK)
	{
		for (i=1;i<7;i++)
		{
			FieldPtr	fldP;
			CharPtr		txtP;
			VoidHand	myhand=DmGetRecord(customDB, i-1);
			customrec	*working=MemHandleLock(myhand),local;

			fldP=FrmGetObjectPtr(form, FrmGetObjectIndex(form, 4000+i));
			txtP=FldGetTextPtr(fldP);
			MemSet(&local, sizeof(local),0);
			StrCopy(local.name, txtP);
			fldP=FrmGetObjectPtr(form, FrmGetObjectIndex(form, 4010+i));
			txtP=FldGetTextPtr(fldP);
			StrCopy(local.number, txtP);
			DmWrite(working, 0, &local, sizeof(local));
			MemHandleUnlock(myhand);
			DmReleaseRecord(customDB, i-1, true);
		}
	}
	FrmSetActiveForm(previousForm);
	FrmDeleteForm(form);
	FrmDrawForm(previousForm);
	DrawCustomButtons(previousForm);
}
static void DrawCustomButtons(FormPtr form)
{
	int i;
				
	for (i=1;i<7;i++)
	{
		Word	buttonindex=FrmGetObjectIndex(form, custombase+i);
		VoidHand myhand=DmQueryRecord(customDB, i-1);
		VoidPtr	 buttonptr=FrmGetObjectPtr(form, buttonindex);
		customrec *working=MemHandleLock(myhand);

		FrmHideObject(form, buttonindex);
		CtlSetLabel(buttonptr, working->name);
		FrmShowObject(form, buttonindex);
		MemHandleUnlock(myhand);
	}
}

static void DoAbout()
{
	FormPtr form = FrmGetActiveForm();
	FormPtr aboutform = FrmInitForm(formID_about);
	Word hitButton;

	FrmSetActiveForm(aboutform);
	hitButton=FrmDoDialog(aboutform);
	FrmSetActiveForm(form);
	FrmDeleteForm(aboutform);
	FrmDrawForm(form);
}

static void PlayClip()
{
	Word cliplength;
	VoidHand cliphand, lochand;
	CharPtr locstr, clipstr;

	cliphand=ClipboardGetItem(clipboardText, &cliplength);
	if (cliplength==0) return;

	lochand=MemHandleNew(cliplength+1);
	locstr=MemHandleLock(lochand);
	clipstr=MemHandleLock(cliphand);
	StrNCopy(locstr, clipstr, cliplength);
	MemHandleUnlock(cliphand);
	if (TRGPro) AudioPlayDTMFStr(libRef, locstr, 2, 3);
	MemHandleUnlock(lochand);
	MemHandleFree(lochand);
}

static void DoAddressDial()
{
	FormPtr form=FrmGetActiveForm();
	FieldPtr fld=FrmGetObjectPtr(form, FrmGetObjectIndex(form, fieldID_address));
	VoidHand storh=MemHandleNew(FldGetTextLength(fld)+1);
	CharPtr storage=MemHandleLock(storh);
	CharPtr numstore;
	CharPtr fldptr;
	Int i, j=0;

	if (FldGetTextLength(fld)==0)
	{
		MemHandleUnlock(storh);
		MemHandleFree(storh);
		return;
	}
	StrCopy(storage, FldGetTextPtr(fld));
	MemHandleUnlock(storh);
	PhoneNumberLookup(fld);
	numstore=(CharPtr)MemPtrNew(FldGetTextLength(fld)+1);
	fldptr=FldGetTextPtr(fld);
	for (i=0;i<StrLen(fldptr);i++)
	{
		if ((fldptr[i]>='0') &&  (fldptr[i]<='9'))
		{
			numstore[j]=fldptr[i];
			j++;
		}
	}
	numstore[j]=0;
	FldSetTextHandle(fld, (Handle) storh);
	FldDrawField(fld);
	if (TRGPro) AudioPlayDTMFStr(libRef, numstore, 2, 3);
	FldSetTextHandle(fld, (Handle) storh);
	FldDrawField(fld);
	MemPtrFree(numstore);
}
